dnl ***************************************************************************
dnl
dnl	$Id: aclocal.m4,v 1.1 2024/10/20 01:45:28 warme Exp $
dnl
dnl	File:	aclocal.m4
dnl	Rev:	a-2
dnl	Date:	10/19/2024
dnl
dnl	Copyright (c) 2012, 2024 by David M. Warme.  All rights
dnl	reserved.  This work is licensed under a Creative Commons
dnl	Attribution-NonCommercial-ShareAlike 4.0 International License.
dnl
dnl ***************************************************************************
dnl
dnl	Autoconf macros for CALIB.
dnl
dnl ***************************************************************************
dnl
dnl	Modification Log:
dnl
dnl	a-1:	10/10/2012	warme
dnl		: Created.
dnl	a-2:	10/19/2024	warme
dnl		: Cleanup for initial release.
dnl
dnl ***************************************************************************

dnl----------------------------------------------------------------------
dnl  GMP
dnl----------------------------------------------------------------------
AC_DEFUN(AC_GMP,
[dnl set up to use GMP
if test -z "$GMP_LIBS"
then
	AC_TRY_CPP([#include <gmp.h>],
	[# We can compile using <gmp.h> with no special include dir.
	GMP_LIBS=-lgmp
	])
fi
AC_MSG_NOTICE([GMP_LIBS=$GMP_LIBS])
AC_SUBST(GMP_LIBS)
])

dnl----------------------------------------------------------------------
dnl  MPFR
dnl----------------------------------------------------------------------
AC_DEFUN(AC_MPFR,
[dnl set up to use MPFR
if test -z "$MPFR_LIBS"
then
	AC_TRY_CPP([#include <mpfr.h>],
	[# We can compile using <mpfr.h> with no special include dir.
	MPFR_LIBS=-lmpfr
	])
fi
AC_MSG_NOTICE([MPFR_LIBS=$MPFR_LIBS])
AC_SUBST(MPFR_LIBS)
])


dnl AC_DEFINE_EXPAND_VALUE(VARIABLE [, VALUE])
AC_DEFUN(AC_DEFINE_EXPAND_VALUE,
[dnl call AC_DEFINE after expanding given value
ac_expand_text1=""
ac_expand_text2="`eval echo \"$2\"`"
while test "$ac_expand_text1" != "$ac_expand_text2"
do
	ac_expand_text3="`eval echo \"$ac_expand_text2\"`"
	ac_expand_text1="$ac_expand_text2"
	ac_expand_text2="$ac_expand_text3"
done
AC_DEFINE_UNQUOTED([$1], "$ac_expand_text1")
])

dnl AC_FIND_FILE(VARIABLE, FILENAME, TESTFLAG, DIRS)
AC_DEFUN(AC_FIND_FILE,
[dnl loop over the dirs, until found
  $1=''
  for cv_ff_dir in $4
  do
        if test "$3" "$cv_ff_dir/$2"
        then
                $1="$cv_ff_dir/$2"
                break
        fi
  done
])

dnl AC_OLD_SYSTEM_WORKAROUND
AC_DEFUN(AC_OLD_SYSTEM_WORKAROUND,
[dnl
AC_ARG_ENABLE(oldsystem,
[  --enable-oldsystem  Work around issues on very old systems],
[if test "$enableval" != "yes"
 then
   AC_MSG_ERROR([--enable-oldsystem does not accept =value.])
 fi
 AC_DEFINE(CALIB_OLD_SYSTEM_WORKAROUND)
])
])

dnl AC_CHECK_WERROR
AC_DEFUN(AC_CHECK_WERROR,
[dnl
WERROR_FLAG=''
AC_ARG_ENABLE([Werror],
	    [--enable-Werror - Enable -Werror.],
[if test "${enableval}" != "yes"
 then
   AC_MSG_ERROR([--enable-Werror does not accept =value.])
 fi
 WERROR_FLAG='-Werror'
])
AC_SUBST(WERROR_FLAG)
])

dnl AC_CHECK_FPLLL
AC_DEFUN(AC_CHECK_FPLLL,
[dnl
AC_ARG_WITH([fplllheaderdir],
	    [--with-fplllheaderdir=dir - dir containing FPLLL header files.])
fphdir="$with_fplllheaderdir"
FPLLL_HEADERDIR=""
if test "$fphdir" != ""
then
	# --with-fplllheaderdir=foo was specified.
	first_char="`echo \"$fphdir\" | sed -e 's/^\(.\).*$/\1/'`"
	if test "$first_char" != "/"
	then
		fphdir="`pwd`/$fphdir"
	fi
	if test -d "$fphdir"
	then
		true
	else
		AC_MSG_ERROR([--with-fplllheaderdir: $fphdir: No such directory.])
	fi
	if test -r "$fphdir/fplll/wrapper.h"
	then
		true
	else
		AC_MSG_ERROR([--with-fplllheaderdir: Can't find fplll/wrapper.h in $fphdir])
	fi
	FPLLL_HEADERDIR="$fphdir"
fi
AC_ARG_WITH([fpllllib],
	    [--with-fpllllib=path - pathname of FPLLL library.])
fplib="$with_fpllllib"
FPLLL_LIB=""
if test "$fplib" != ""
then
	# --with-fpllllib was specified.
	first_char="`echo \"$fplib\" | sed -e 's/^\(.\).*$/\1/'`"
	if test "$first_char" != "/"
	then
		fplib="`pwd`/$fplib"
	fi
	if test -r "$fplib"
	then
		true
	else
		AC_MSG_ERROR([--with-fpllllib: $fplib: Library unreadable.])
	fi
	FPLLL_LIB="$fplib"
fi
if test "$FPLLL_HEADERDIR" != ""
then
	if test "$FPLLL_LIB" != ""
	then
		# Have both headerdir and lib!
		# For calib/config.h:
		AC_DEFINE(CALIB_HAVE_FPLLL)

		# For Minit.mk:
		HAVE_FPLLL=yes
		AC_SUBST(HAVE_FPLLL)
		AC_SUBST(FPLLL_HEADERDIR)
		AC_SUBST(FPLLL_LIB)
	fi
fi
])
