/**********************************************************************

	$Id: Zp.h,v 1.1 2024/10/20 00:09:10 warme Exp $

	File:	Zp.h
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2012, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	The field of integers modulo p, where p is a prime
	of at most 32 bits.  The "object" is a GMP integer.

***********************************************************************

	Modification Log:

	a-1:	04/10/2012	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#ifndef CALIB_ZP_H
#define CALIB_ZP_H

#include "calib/types.h"
#include <gmp.h>

#ifdef __cplusplus
 extern "C" {
#endif

struct calib_genrep;
struct calib_Random;
struct calib_Zp_dom;

/* The field of integers modulo p. */

struct calib_Zp_dom {
	mpz_t	p;		/* Prime modulus */
	void	(*set_si) (const struct calib_Zp_dom *	f,
			  mpz_ptr			rop,
			  calib_si_t			op);
	void	(*set_z) (const struct calib_Zp_dom *	f,
			mpz_ptr				rop,
			mpz_srcptr			op);
	void	(*set_q) (const struct calib_Zp_dom *	f,
			  mpz_ptr			rop,
			  mpq_srcptr			op);
	void	(*add) (const struct calib_Zp_dom *	f,
			mpz_ptr				rop,
			mpz_srcptr			op1,
			mpz_srcptr			op2);
	void	(*sub) (const struct calib_Zp_dom *	f,
			mpz_ptr				rop,
			mpz_srcptr			op1,
			mpz_srcptr			op2);
	void	(*neg) (const struct calib_Zp_dom *	f,
			mpz_ptr				rop,
			mpz_srcptr			op);
	void	(*mul) (const struct calib_Zp_dom *	f,
			mpz_ptr				rop,
			mpz_srcptr			op1,
			mpz_srcptr			op2);
	void	(*ipow) (const struct calib_Zp_dom *	f,
			 mpz_ptr			rop,
			 mpz_srcptr			op,
			 int				power);
	void	(*inv) (const struct calib_Zp_dom *	f,
			mpz_ptr				rop,
			mpz_srcptr			op);
	void	(*set_random) (const struct calib_Zp_dom *	f,
			       mpz_ptr				rop,
			       struct calib_Random *		randp);
	void	(*set_genrep) (const struct calib_Zp_dom *	f,
			       mpz_ptr				rop,
			       const struct calib_genrep *	op);
	struct calib_genrep *
		(*to_genrep) (const struct calib_Zp_dom *	f,
			      mpz_srcptr			op);
};

/* Construct/initialize the given Zp arithmetic domain. */

extern void	calib_free_Zp_dom (struct calib_Zp_dom * cf);
extern struct calib_Zp_dom *
		calib_make_Zp_dom (mpz_srcptr p);
extern struct calib_Zp_dom *
		calib_make_Zp_dom_ui (unsigned long p);

#ifdef __cplusplus
 }
#endif

#endif
