/**********************************************************************

	$Id: fatal.h,v 1.1 2024/10/20 00:09:11 warme Exp $

	File:	fatal.h
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2012, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	Fatal errors.

***********************************************************************

	Modification Log:

	a-1:	04/10/2012	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#ifndef CALIB_FATAL_H
#define CALIB_FATAL_H

#ifdef __cplusplus
 extern "C" {
#endif

/*
 * Macros for issuing fatal errors.
 */

#define CALIB_FATAL_ERROR \
	do { \
		_calib_fatal (__FILE__, __LINE__); \
	} while (0)

#define CALIB_FATAL_ERROR_IF(condition) \
	do { \
		if (condition) { \
			_calib_fatal (__FILE__, __LINE__); \
		} \
	} while (0)

#ifdef __GNUC__
  #define __NORETURN__	__attribute__ ((noreturn))
#else
  #define __NORETURN__
#endif

extern void		_calib_fatal (const char * filename, int lineno)
				__NORETURN__;

#ifdef __cplusplus
 }
#endif

#endif
