/**********************************************************************

	$Id: gmpmisc.h,v 1.1 2024/10/20 00:09:11 warme Exp $

	File:	gmpmisc.h
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2012, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	Miscellaneous extensions and utilities for GMP.

***********************************************************************

	Modification Log:

	a-1:	04/10/2012	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#ifndef CALIB_GMPMISC_H
#define CALIB_GMPMISC_H

#include <calib/types.h>
#include <gmp.h>
#include <stddef.h>

#ifdef __cplusplus
 extern "C" {
#endif

/* Functions we wish were in GMP. */
extern void	calib_mpq_set_z_z (mpq_ptr	rop,
				   mpz_srcptr	op1,
				   mpz_srcptr	op2);
extern void	calib_mpq_div_si (mpq_ptr	rop,
				  mpq_srcptr	op1,
				  calib_si_t	op2);
extern void	calib_mpq_div_z (mpq_ptr	rop,
				 mpq_srcptr	op1,
				 mpz_srcptr	op2);
extern void	calib_mpq_mul_si (mpq_ptr	rop,
				  mpq_srcptr	op1,
				  calib_si_t	op2);
extern void	calib_mpq_mul_z (mpq_ptr	rop,
				 mpq_srcptr	op1,
				 mpz_srcptr	op2);

/* Routines for allocating/freeing GMP vectors (Z and Q). */
extern void	calib_clear_Q_vector (mpq_ptr p, size_t n);
extern void	calib_clear_Z_vector (mpz_ptr p, size_t n);
extern void	calib_copy_Q_vector (mpq_ptr dst, mpq_srcptr src, size_t n);
extern void	calib_copy_Z_vector (mpz_ptr dst, mpz_srcptr src, size_t n);
extern void	calib_free_Q_vector (mpq_ptr p, size_t n);
extern void	calib_free_Z_vector (mpz_ptr p, size_t n);
extern void	calib_init_Q_vector (mpq_ptr p, size_t n);
extern void	calib_init_Z_vector (mpz_ptr p, size_t n);
extern mpq_ptr	calib_new_Q_vector (size_t n);
extern mpz_ptr	calib_new_Z_vector (size_t n);
extern void	calib_print_Q_vector (mpq_srcptr p, size_t n);
extern void	calib_print_Z_vector (mpz_srcptr p, size_t n);

extern size_t	calib_mpz_bit_size (mpz_srcptr z);
extern size_t	calib_mpz_bit_size_vector (mpz_srcptr op, size_t n);
extern size_t	calib_mpq_bit_size (mpq_srcptr q);
extern size_t	calib_mpq_bit_size_vector (mpq_srcptr op, size_t n);

#ifdef __cplusplus
 }
#endif

#endif
