/***********************************************************************

	$Id: random.h,v 1.1 2024/10/20 00:09:11 warme Exp $

	File:	random.h
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2006, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

************************************************************************

	A simple random number generator object.

************************************************************************

	Modification Log:

	a-1:	01/31/2006	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

************************************************************************/

#ifndef CALIB_RANDOM_H
#define CALIB_RANDOM_H

#include "calib/types.h"

#ifdef __cplusplus
 extern "C" {
#endif

/* The state information for a simple Pseudo-Random Number Generator. */
/* We use 64-bits of state.  This is enough for our purposes. */

struct calib_Random {
	calib_int32u	lo;
	calib_int32u	hi;
	double		normal_val2;	/* Cache 2nd normal deviate */
	int		normal_flag;	/* val2 is valid iff flag is TRUE */
};

extern void		calib_random_init (struct calib_Random * state);
extern double		calib_random (struct calib_Random * state);
extern double		calib_random_normal (struct calib_Random * state);
extern calib_int32u	calib_random_u32 (struct calib_Random * state);
extern calib_int64u	calib_random_u64 (struct calib_Random * state);

#ifdef __cplusplus
 }
#endif

#endif
