/**********************************************************************

	$Id: Qax_int.c,v 1.1 2024/10/20 00:36:26 warme Exp $

	File:	Qax_int.c
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2012, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	Internal Qax interfaces.

***********************************************************************

	Modification Log:

	a-1:	06/11/2017	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#include "Qax_int.h"

#include "calib/fatal.h"
#include "calib/gmpmisc.h"
#include "calib/logic.h"
#include "calib/new.h"
#include "calib/Qa.h"
#include "calib/Qax.h"
#include <stdlib.h>
#include <string.h>


/*
 * Initialize the given QaxCoeffs object from the given Qax polymonial.
 * This obtains the full rational values for each Q(a) coefficient.
 */

	void
_calib_QaxCoeffs_init (

struct QaxCoeffs *		rop,	/* IN/OUT: rational coefficients */
const struct calib_Qax_obj *	op	/* IN: Polynomial for which to */
					/*     get all the coefficients */
)
{
int				i, deg;
const struct calib_Qax_dom *	K_of_x;
const struct calib_Qa_dom *	cf;
struct QaVal *			qvarray;
const struct calib_Qa_obj *	qap;


	CALIB_FATAL_ERROR_IF ((rop EQ NULL) OR (op EQ NULL));

	deg	= op -> degree;
	K_of_x	= op -> dom;
	cf	= K_of_x -> cf;

	qvarray	= CALIB_NEWA (deg + 1, struct QaVal);

	for (i = 0; i <= deg; i++) {
		qap	= &(op -> coeff [i]);

		qvarray [i].qa_deg	= qap -> degree;
		qvarray [i].qa_coeff	= cf -> get_coeffs (qap);
	}

	rop -> qax_deg		= deg;
	rop -> qax_coeff	= qvarray;
}

/*
 * Clear out the given QaxCoeffs object.
 */

	void
_calib_QaxCoeffs_clear (

struct QaxCoeffs *	op	/* IN/OUT: rational coefficients to clear */
)
{
int		i, deg, deg2;
struct QaVal *	qvarray;
mpq_ptr		qp;

	CALIB_FATAL_ERROR_IF (op EQ NULL);

	deg	= op -> qax_deg;
	qvarray	= op -> qax_coeff;

	for (i = 0; i <= deg; i++) {
		deg2	= qvarray [i].qa_deg;
		qp	= qvarray [i].qa_coeff;

		calib_free_Q_vector (qp, deg2 + 1);
	}
	memset (qvarray, 0, (deg + 1) * sizeof (qvarray [0]));
	free (qvarray);

	memset (op, 0, sizeof (*op));
}
