/**********************************************************************

	$Id: Qax_int.h,v 1.1 2024/10/20 00:36:26 warme Exp $

	File:	Qax_int.h
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2012, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	Internal Qax interfaces.

***********************************************************************

	Modification Log:

	a-1:	06/11/2017	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#ifndef CALIB_QAX_INT_H
#define CALIB_QAX_INT_H

#include <stdio.h>
#include <gmp.h>

struct calib_Qax_obj;


/*
 * An object to represent the rational coefficients of a Q(a) value.
 */

struct QaVal {
	int		qa_deg;		/* Degree of Q(a) value */
	mpq_ptr		qa_coeff;	/* Coefficients of Q(a) value */
};

/*
 * An object to hold all rational coefficients of a Qax polynomial.
 */

struct QaxCoeffs {
	int		qax_deg;	/* Degree of Qax polynomial */
	struct QaVal *	qax_coeff;	/* Q(a) coeffcients */
};

/*
 * Function signatures.  These need _calib prefixes to keep the binary
 * namespace clean.  The above structures don't need to be clean, however,
 * because this header is not made available to applications.
 */

extern void	_calib_QaxCoeffs_clear (struct QaxCoeffs *		op);
extern void	_calib_QaxCoeffs_init (struct QaxCoeffs *		rop,
				       const struct calib_Qax_obj *	op);

#endif
