/**********************************************************************

	$Id: fplll.cc,v 1.1 2024/10/20 00:36:27 warme Exp $

	File:	fplll.cc
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2012, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	Provide C-callable function to access LLL functionality
	of the FPLLL package.  This must be written in C++.

***********************************************************************

	Modification Log:

	a-1:	08/09/2020	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#include "calib/config.h"

#ifdef CALIB_HAVE_FPLLL

#include "fplll/wrapper.h"
#include "fplll/nr/matrix.h"
// #include <config.h>

template <class ZT> int lll (

fplll::ZZ_mat<ZT> &	b	//- IN/OUT: lattice vectors to reduce
)
{
int		status;

	double			delta		= 0.99;
	double			eta		= 0.51;
	fplll::LLLMethod	method		= fplll::LM_WRAPPER;
	fplll::FloatType	float_type	= fplll::FT_DEFAULT;
	int			precision	= 0;
	int			flags		= 0;

	status = fplll::lll_reduction (
				b,
				delta,
				eta,
				method,
				float_type,
				precision,
				flags);

	return status;
}

extern "C" {

	int
calib_FPLLL (

mpz_ptr			dst,		//- OUT: Reduced basis
mpz_srcptr		src,		//- IN: Input basis
int			nvec,		//- IN: Number of vectors
int			vsize		//- IN: Size of each vector
)
{
int			i, j, result;
fplll::ZZ_mat<mpz_t>	m (nvec, vsize);
mpz_t			tmp;

	mpz_init (tmp);

	for (i = 0; i < nvec; i++) {
		for (j = 0; j < vsize; j++) {
			mpz_set (tmp, &src [i * vsize + j]);
			m [i] [j] = tmp;
		}
	}

	result = lll<mpz_t> (m);

	for (i = 0; i < nvec; i++) {
		for (j = 0; j < vsize; j++) {
			m [i] [j].get_mpz (tmp);
			mpz_set (&dst [i * vsize + j], tmp);
		}
	}

	mpz_clear (tmp);

	return result;
}

}	// End of extern "C"

#endif
