/**********************************************************************

	$Id: genrep_int.h,v 1.1 2024/10/20 00:36:27 warme Exp $

	File:	genrep_int.h
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2012, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	Internal routines for general representation for expressions.

***********************************************************************

	Modification Log:

	a-1:	06/11/2017	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#ifndef CALIB_GENREP_INT_H
#define CALIB_GENREP_INT_H

#include <stdio.h>
#include <gmp.h>

/*
 * A structure for accumulating a sequence of operands, with a place
 * to handle constant rational operands specially.
 */

struct _genrep_nary {
	mpq_t				qval;	/* Constant value */
	struct calib_genrep_list *	list;	/* List of operands */
	struct calib_genrep_list **	hookp;	/* Where to append next */
						/* operand */
};

typedef struct calib_genrep * (*_genrep_dup_func) (const struct calib_genrep * op);

/*
 * Function signatures.
 */

struct calib_genrep *
		_calib_genrep_copy_simp (const struct calib_genrep * op);
struct calib_genrep *
		_calib_genrep_finish_prod (struct _genrep_nary * root);
struct calib_genrep *
		_calib_genrep_finish_sum (struct _genrep_nary * root);
void		_calib_genrep_init_prod (struct _genrep_nary * root);
void		_calib_genrep_init_sum (struct _genrep_nary * root);
void		_calib_genrep_insert_prod (
				struct _genrep_nary *		root,
				const struct calib_genrep *	op,
				_genrep_dup_func		dup_func);
void		_calib_genrep_insert_sum (
				struct _genrep_nary *		root,
				const struct calib_genrep *	op,
				_genrep_dup_func		dup_func);

#endif
