/**********************************************************************

	$Id: modcleanup.c,v 1.1 2024/10/20 00:36:28 warme Exp $

	File:	modcleanup.c
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2012, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	Internal interfaces to cleanup the algebra library when done.

***********************************************************************

	Modification Log:

	a-1:	10/16/2015	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#include "modcleanup.h"

#include "calib/fatal.h"
#include "calib/logic.h"
#include "calib/new.h"
#include <stdlib.h>

struct cleanup {
	calib_module_cleanup_func_t *	handler;
	struct cleanup *		next;
};

static struct cleanup *	global_cleanup_handler_list;


/*
 * Internal routine (should not be called by applications).
 *
 * Add the given handler to our internal list of cleanup activities.
 */

	void
_calib_add_cleanup_handler (

calib_module_cleanup_func_t *	handler	/* IN: cleanup handler function */
)
{
struct cleanup *	p;

	/* Make sure this handler is not already specified. */
	for (p = global_cleanup_handler_list; p NE NULL; p = p -> next) {
		CALIB_FATAL_ERROR_IF (p -> handler EQ handler);
	}

	p = CALIB_NEW (struct cleanup);
	p -> handler	= handler;
	p -> next	= global_cleanup_handler_list;

	global_cleanup_handler_list = p;
}


/*
 * The routine that applications should call when they want to clean up
 * this library.
 */

	void
calib_shutdown (void)

{
struct cleanup *	p;
struct cleanup *	p2;

	p = global_cleanup_handler_list;
	global_cleanup_handler_list = NULL;

	while (p NE NULL) {
		p2 = p -> next;
		p -> next = NULL;

		p -> handler ();

		free (p);

		p = p2;
	}

	/* New handlers sprouted during cleanup!?!? */
	CALIB_FATAL_ERROR_IF (global_cleanup_handler_list NE NULL);
}
