/**********************************************************************

	$Id: oldsystem.cc,v 1.1 2024/10/20 00:36:28 warme Exp $

	File:	oldsystem.cc
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2007, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	A stub for handling problems with old operating systems.

***********************************************************************

	Modification Log:

	a-1:	08/09/2020	warme
		: Created from earlier stand-alone code.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#include "calib/config.h"

#ifdef	CALIB_OLD_SYSTEM_WORKAROUND

/*
 * MPFR has defined two different functions for converting an
 *  mpfr_t to a mpz_t with "long long" exponent:
 *
 *	mpfr_get_z_exp		(old)
 *
 *	mpfr_get_z_2exp		(current)
 *
 * The problem is that modern versions of MPFR do the following
 * (for backward "compatibility"):
 *
 *	#define	mpfr_get_z_exp		mpfr_get_z_2exp
 *
 * Unfortunately, this produces object files that always refer to the
 * "new" function.  That object file does not work when linked with
 * older versions of MPFR.
 *
 * This is not usually a problem.  But I think I had to compile FPLLL
 * on a "new" system and copy the objects to old system "dmw" (CentOS 6.10)
 * (because FPLLL does not compile there).  The FPLLL object files refer
 * to the newer mpfr_get_z_2exp function which is not provided by
 * the old CentOS 6 MPFR.  The C++ runtime also does not provide certain
 * runtime_error functions, so we provide stubs for them here.
 *
 * This is purely a workaround to get this to work on CentOS 6.  It
 * should not be needed for newer systems.
 */

#include <cstdlib>
#include <mpfr.h>

extern "C" {

void	_ZNSt13runtime_errorC1EPKc ()	{ abort (); }
void	_ZNSt13runtime_errorD1Ev()	{ abort (); }
void	_ZTISt13runtime_error ()	{ abort (); }

	mp_exp_t
mpfr_get_z_2exp (

mpz_ptr		rop,		/* IN/OUT: Mantissa value */
mpfr_srcptr	op		/* IN: MPFR value */
)
{
	return (mpfr_get_z_exp (rop, op));
}

}

#endif
