/**********************************************************************

	$Id: prompt.c,v 1.1 2024/10/20 00:36:28 warme Exp $

	File:	prompt.c
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2012, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	A routine to print prompts -- but only if stdin is a tty.

***********************************************************************

	Modification Log:

	a-1:	10/08/2012	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#include "calib/prompt.h"

#include "calib/logic.h"
#include <stddef.h>
#include <stdio.h>
#include <unistd.h>


/*
 * Issue the given prompt string -- but only if stdin is connected
 * to a tty.
 */

	void
calib_prompt (

const char *	str		/* IN: prompt string to issue */
)
{

	static int is_tty = -1;
	if (is_tty < 0) {
		is_tty = isatty (fileno (stdin));
	}

	if (is_tty) {
		printf ("%s", str);
		fflush (stdout);
	}
}
