/**********************************************************************

	$Id: ratfac.h,v 1.1 2024/10/20 00:36:28 warme Exp $

	File:	ratfac.h
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2012, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	Rational functions in partially-factored form.

	Note: This is currently used only in Zxyz::resultant_old()
	-- an old, slow, obsolete multi-variate polynomial resultant
	algorithm.

***********************************************************************

	Modification Log:

	a-1:	08/01/2020	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#ifndef CALIB_RATFAC_H
#define CALIB_RATFAC_H

struct calib_RFImp;
struct calib_Zxyz_dom;
struct calib_Zxyz_obj;


/*
 * A partially-factored rational function.  Implementation is hidden.
 */

struct calib_RatFac {
	struct calib_RFImp *	rf_imp;		/* Implementation object */
};


/*
 * Function signatures.
 */

/* Init and clear. */
extern void	_calib_RatFac_init (const struct calib_Zxyz_dom *	dom,
				    struct calib_RatFac *		rfp);
extern void	_calib_RatFac_clear (struct calib_RatFac * rfp);

/* Multiply this ratfac by poly(x,y,z)**pow, where pow is signed integer. */
extern void	_calib_RatFac_mul (struct calib_RatFac *		rfp,
				   const struct calib_Zxyz_obj *	poly,
				   int					pow);

extern void	_calib_RatFac_negate (struct calib_RatFac * rfp);

/* Print the given RatFac object (for debugging). */
extern void	_calib_RatFac_print (const struct calib_RatFac * rfp);

/* Return partially factored numerator. */
extern struct calib_Zxyz_factor *
		_calib_RatFac_numer (const struct calib_RatFac * rfp);

/* Return partially factored denominator. */
extern struct calib_Zxyz_factor *
		_calib_RatFac_denom (const struct calib_RatFac * rfp);

#endif
