/**********************************************************************

	$Id: test_qx_factor.c,v 1.1 2024/10/20 01:01:00 warme Exp $

	File:	test_qx_factor.c
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2012, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	Test implementation of Q[x].

***********************************************************************

	Modification Log:

	a-1:	05/03/2012	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#include "calib/genrep.h"
#include "calib/logic.h"
#include "calib/prompt.h"
#include "calib/Qx.h"
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

	int
main ()

{
int				status;
struct calib_genrep *		gp1;
struct calib_genrep *		gp2;
char *				var;
struct calib_Qx_obj		POLY;
struct calib_Qx_factor *	factors;
const struct calib_Qx_dom *	Q_of_x;
struct calib_genrep_varlist	varlist;

	/* Get the polynomial ring Q[x]. */
	Q_of_x = calib_get_Qx_dom ();

	Q_of_x -> init (&POLY);

	while (1) {
		calib_prompt ("Polynomial: ");
		gp1 = calib_genrep_read (&status);
		if (status NE 0) {
			fprintf (stderr, "Syntax error\n");
			continue;
		}
		if (gp1 EQ NULL) break;

		/* Verify exactly one variable. */
		calib_genrep_get_varlist (&varlist, gp1);

		var = NULL;
		if (varlist.nvars EQ 1) {
			var = strdup (varlist.vlist [0]);
		}
		else if (varlist.nvars EQ 0) {
			var = strdup ("x");
		}
		calib_genrep_clear_varlist (&varlist);

		if (var EQ NULL) {
			fprintf (stderr,
				 "Expression must use exactly"
				 " one variable.\n");
			calib_genrep_free (gp1);
			continue;
		}

		Q_of_x -> set_genrep (&POLY, gp1, var);
		calib_genrep_free (gp1);
		if (POLY.degree < 0) {
			free (var);
			break;
		}

		/* Take canonical form back into general representation. */
		gp1 = Q_of_x -> to_genrep (&POLY, var);

		factors = Q_of_x -> factor (&POLY);

		gp2 = Q_of_x -> factors_to_genrep (factors, var);

		printf ("\n");
		calib_genrep_print (gp1);
		printf (" =\n");
		calib_genrep_print (gp2);
		printf ("\n");

		calib_genrep_free (gp2);
		Q_of_x -> free_factors (factors);
		calib_genrep_free (gp1);
		free (var);
	}

	Q_of_x -> clear (&POLY);

	exit (0);
}
