/**********************************************************************

	$Id: test_zpx_factor.c,v 1.1 2024/10/20 01:04:36 warme Exp $

	File:	test_zpx_factor.c
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2012, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	Test factorization of polynomials over Zp[x].

***********************************************************************

	Modification Log:

	a-1:	04/10/2012	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#include "calib/genrep.h"
#include "calib/logic.h"
#include "calib/prompt.h"
#include "calib/Zpx.h"
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

	int
main ()

{
int				status;
int				modulus;
struct calib_genrep *		gp1;
struct calib_genrep *		gp2;
char *				var;
struct calib_Zpx_factor *	factors;
struct calib_Zp_dom *		Zp;
struct calib_Zpx_dom *		K_of_x;
struct calib_Zpx_obj		POLY;
struct calib_genrep_varlist	varlist;

	while (1) {
		calib_prompt ("Modulus: ");
		(void) scanf (" %d", &modulus);
		if (modulus < 2) break;

		/* Construct our coefficient number field. */
		Zp = calib_make_Zp_dom_ui (modulus);

		/* Construct polynomial ring. */
		K_of_x = calib_make_Zpx_dom (Zp);

		calib_prompt ("Polynomial: ");
		gp1 = calib_genrep_read (&status);
		if (status NE 0) {
			fprintf (stderr, "Syntax error\n");
			calib_free_Zpx_dom (K_of_x);
			calib_free_Zp_dom (Zp);
			continue;
		}
		if (gp1 EQ NULL) break;

		/* Verify exactly one variable. */
		calib_genrep_get_varlist (&varlist, gp1);

		var = NULL;
		if (varlist.nvars EQ 1) {
			var = strdup (varlist.vlist [0]);
		}
		calib_genrep_clear_varlist (&varlist);

		if (var EQ NULL) {
			fprintf (stderr,
				 "Expression must use exactly"
				 " one variable.\n");
			calib_genrep_free (gp1);
			calib_free_Zpx_dom (K_of_x);
			calib_free_Zp_dom (Zp);
			continue;
		}

		K_of_x -> init (K_of_x, &POLY);

		K_of_x -> set_genrep (&POLY, gp1, var);

		/* Take canonical form back into general representation. */
		calib_genrep_free (gp1);
		gp1 = K_of_x -> to_genrep (&POLY, var);

		factors = K_of_x -> factor (&POLY);

		gp2 = K_of_x -> factors_to_genrep (factors, var);

		printf ("\n");
		calib_genrep_print (gp1);
		printf (" =\n");
		calib_genrep_print (gp2);
		printf ("\n");

		calib_genrep_free (gp2);
		K_of_x -> free_factors (factors);
		K_of_x -> clear (&POLY);
		free (var);
		calib_genrep_free (gp1);
		calib_free_Zpx_dom (K_of_x);
		calib_free_Zp_dom (Zp);
	}
	exit (0);
}
