/**********************************************************************

	$Id: test_foo.c,v 1.1 2024/10/20 01:24:49 warme Exp $

	File:	test_foo.c
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2020, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	Test driver to debug a specific problem.

***********************************************************************

	Modification Log:

	a-1:	07/11/2020	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#include "calib/calib.h"
#include "calib/cputime.h"
#include "calib/logic.h"
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

static void add_term (struct calib_Zx_obj * poly, int c, int i);

	int
main ()

{
const struct calib_Zx_dom *	dom;
const struct calib_Zx_obj *	polys [2];
struct calib_Zx_obj *		cofact [2];
struct calib_Zx_obj		A, B, GCD, CFA, CFB;
struct calib_Zx_obj		FOO, FOO2, FOO3, BAR;

	dom = calib_get_Zx_dom ();

	dom -> init (&A);
	dom -> init (&B);
	dom -> init (&GCD);
	dom -> init (&CFA);
	dom -> init (&CFB);
	dom -> init (&FOO);
	dom -> init (&FOO2);
	dom -> init (&FOO3);
	dom -> init (&BAR);

	/* Generate FOO = x^16 - x^8 + x^4 - 1. */
	add_term (&FOO,  1, 16);
	add_term (&FOO, -1, 8);
	add_term (&FOO,  1, 4);
	add_term (&FOO, -1, 0);

	/* Generate BAR = x^24 - x^20 - x^4 + 1. */
	add_term (&BAR,  1, 24);
	add_term (&BAR, -1, 20);
	add_term (&BAR, -1, 4);
	add_term (&BAR,  1, 0);

	dom -> mul (&FOO2, &FOO, &FOO);
	dom -> mul (&FOO3, &FOO2, &FOO);

	/* Generate A = BAR * FOO^2. */
	dom -> mul (&A, &BAR, &FOO2);

	/* Generate B = FOO^3. */
	dom -> set (&B, &FOO3);

	polys [0]	= &A;
	polys [1]	= &B;
	cofact [0]	= &CFA;
	cofact [1]	= &CFB;

	printf ("A = ");
	dom -> print_maxima (&A);
	printf ("B = ");
	dom -> print_maxima (&B);

	dom -> gcd_n (&GCD, cofact, 2, polys);

	printf ("GCD = ");
	dom -> print_maxima (&GCD);
	printf ("A_COFACTOR = ");
	dom -> print_maxima (&CFA);
	printf ("B_COFACTOR = ");
	dom -> print_maxima (&CFB);

	dom -> clear (&BAR);
	dom -> clear (&FOO3);
	dom -> clear (&FOO2);
	dom -> clear (&FOO);
	dom -> clear (&CFB);
	dom -> clear (&CFA);
	dom -> clear (&GCD);
	dom -> clear (&B);
	dom -> clear (&A);

	/* No need to free Z[x] dom. */

	calib_shutdown ();

	exit (0);
}

/*
 * Add the term c*x**i to the given Z[x] polynomial.
 */

	static
	void
add_term (

struct calib_Zx_obj *	poly,		/* IN/OUT: Poly to add into */
int			c,		/* IN: Coefficient */
int			i		/* IN: Exponent */
)
{
const struct calib_Zx_dom *	dom;
struct calib_Zx_obj		TERM;

	dom = calib_get_Zx_dom ();

	dom -> init (&TERM);
	dom -> alloc (&TERM, i);
	TERM.degree = i;
	mpz_set_si (&TERM.coeff [i], c);

	dom -> add (poly, poly, &TERM);

	dom -> clear (&TERM);
}
