/**********************************************************************

	$Id: test_zx_gcd_n.c,v 1.1 2024/10/20 01:24:49 warme Exp $

	File:	test_zx_gcd_n.c
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2012, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	Test Z[x] gcd_n().

***********************************************************************

	Modification Log:

	a-1:	09/11/2012	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#include "calib/genrep.h"
#include "calib/logic.h"
#include "calib/new.h"
#include "calib/prompt.h"
#include "calib/Zx.h"
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

	int
main ()

{
int				i, status, npoly;
struct calib_genrep *		gp1;
char *				var;
char *				var2;
const struct calib_Zx_obj **	pptrs;
struct calib_Zx_obj *		cfarray;
struct calib_Zx_obj **		cfptrs;
const struct calib_Zx_dom *	Zx;
struct calib_Zx_obj		POLY1;
struct calib_Zx_obj		GCD;
struct calib_genrep_varlist	varlist;
char				buffer [32];

	/* Get the polynomial ring Z[x]. */
	Zx = calib_get_Zx_dom ();

	Zx -> init (&POLY1);
	Zx -> init (&GCD);

	while (1) {
		calib_prompt ("Number of polynomials: ");
		npoly = 0;
		if (scanf (" %d", &npoly) NE 1) break;
		if (npoly < 0) break;

		pptrs = CALIB_NEWA (npoly, const struct calib_Zx_obj *);

		status = 0;
		var = NULL;
		for (i = 0; i < npoly; ) {
			sprintf (buffer, "Polynomial %d: ", i + 1);
			calib_prompt (buffer);
			gp1 = calib_genrep_read (&status);
			if (status NE 0) {
				fprintf (stderr, "Syntax error\n");
				continue;
			}
			if (gp1 EQ NULL) {
				exit (0);
			}

			/* Verify exactly one variable. */
			calib_genrep_get_varlist (&varlist, gp1);

			var2 = NULL;
			if (varlist.nvars EQ 1) {
				var2 = strdup (varlist.vlist [0]);
			}
			else if (varlist.nvars > 1) {
				fprintf (stderr,
					 "Expressions must use exactly"
					 " one variable.\n");
				calib_genrep_free (gp1);
			}
			calib_genrep_clear_varlist (&varlist);

			if ((var NE NULL) AND
			    (var2 NE NULL) AND
			    (strcmp (var, var2) NE 0)) {
				fprintf (stderr,
					 "Expressions must use exactly"
					 " one variable.\n");
				calib_genrep_free (gp1);
				continue;
			}
			else if ((var EQ NULL) AND (var2 NE NULL)) {
				var = strdup (var2);
			}
			if (var2 NE NULL) {
				free (var2);
			}

			Zx -> set_genrep (&POLY1, gp1, var);
			calib_genrep_free (gp1);

			pptrs [i] = Zx -> dup (&POLY1);
			++i;
		}

		/* Create cofactor array. */
		cfarray = CALIB_NEWA (npoly, struct calib_Zx_obj);
		cfptrs	= CALIB_NEWA (npoly, struct calib_Zx_obj *);
		for (i = 0; i < npoly; i++) {
			Zx -> init (&cfarray [i]);
			cfptrs [i] = &cfarray [i];
		}

		Zx -> gcd_n (&GCD, cfptrs, npoly, pptrs);

		printf ("\n");
		printf ("GCD is\n");
		gp1 = Zx -> to_genrep (&GCD, var);
		calib_genrep_print (gp1);
		calib_genrep_free (gp1);

		for (i = 0; i < npoly; i++) {
			printf ("\n");
			printf (" Cofactor %d is\n", i + 1);
			gp1 = Zx -> to_genrep (&cfarray [i], var);
			calib_genrep_print (gp1);
			calib_genrep_free (gp1);
		}
		printf ("\n");

		for (i = 0; i < npoly; i++) {
			Zx -> clear (&cfarray [i]);
			Zx -> free ((struct calib_Zx_obj *) (pptrs [i]));
		}
		free (cfptrs);
		free (pptrs);
		if (var NE NULL) {
			free (var);
		}
	}

	Zx -> clear (&GCD);
	Zx -> clear (&POLY1);

	exit (0);
}
