/**********************************************************************

	$Id: test_zxyz_factor.c,v 1.1 2024/10/20 01:31:30 warme Exp $

	File:	test_zxyz_factor.c
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2012, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	Test Z[x,y,z] factor().

***********************************************************************

	Modification Log:

	a-1:	09/26/2012	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#include "calib/cputime.h"
#include "calib/genrep.h"
#include "calib/logic.h"
#include "calib/prompt.h"
#include "calib/Zxyz.h"
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

	int
main ()

{
int				status;
struct calib_genrep *		gp1;
const struct calib_Zxyz_dom *	Zxyz;
struct calib_Zxyz_factor *	factors;
struct calib_Zxyz_factor *	fp;
calib_cpu_time_t		T0, T1;
char				tbuf [32];
struct calib_genrep_varlist	varlist;
struct calib_Zxyz_obj		POLY, TEMP1, TEMP2, TEMP3;

	while (1) {
		calib_prompt ("Polynomial: ");
		gp1 = calib_genrep_read (&status);
		if (status NE 0) {
			fprintf (stderr, "Syntax error\n");
			continue;
		}
		if (gp1 EQ NULL) break;

		calib_genrep_get_varlist (&varlist, gp1);

		/* Make the multi-variate domain. */
		Zxyz = calib_make_Zxyz_dom (varlist.nvars, varlist.vlist);

		calib_genrep_clear_varlist (&varlist);

		Zxyz -> init (Zxyz, &POLY);

		Zxyz -> set_genrep (&POLY, gp1);
		calib_genrep_free (gp1);

		if (POLY.nterms < 200) {
			/* Take canonical form back into general rep. */
			gp1 = Zxyz -> to_genrep (&POLY);

			printf ("\n");
			calib_genrep_print (gp1);
			printf ("\n");
			calib_genrep_free (gp1);
		}
		else {
			printf ("\n\t{polynomial has %d terms.}\n\n",
				POLY.nterms);
		}

		T0 = calib_get_cpu_time ();
		factors = Zxyz -> factor (&POLY);
		T1 = calib_get_cpu_time ();

		gp1 = Zxyz -> factors_to_genrep (factors);

		printf (" =\n");
		calib_genrep_prettyprint (gp1);
		printf ("\n");

		calib_genrep_free (gp1);

		/* Multiply the factors out to see if they are correct. */
		Zxyz -> init (Zxyz, &TEMP1);
		Zxyz -> init (Zxyz, &TEMP2);
		Zxyz -> init (Zxyz, &TEMP3);
		Zxyz -> set_si (&TEMP1, 1);
		for (fp = factors; fp NE NULL; fp = fp -> next) {
			Zxyz -> ipow (&TEMP2, fp -> factor, fp -> multiplicity);
			Zxyz -> mul (&TEMP1, &TEMP1, &TEMP2);
		}
		Zxyz -> sub (&TEMP3, &POLY, &TEMP1);
		if (TEMP3.nterms <= 0) {
			printf ("\nFactors multiply out correctly\n");
		}
		else {
			Zxyz -> add (&TEMP3, &POLY, &TEMP1);
			if (TEMP3.nterms <= 0) {
				printf ("\n*** SIGN OF RESULT IS WRONG! ***\n");
			}
			else {
				printf ("\n*** FACTORS DO NOT MULTIPLY OUT"
					" CORRECTLY ***\n");
			}
		}
		Zxyz -> clear (&TEMP3);
		Zxyz -> clear (&TEMP2);
		Zxyz -> clear (&TEMP1);

		Zxyz -> free_factors (factors);
		Zxyz -> clear (&POLY);
		calib_free_Zxyz_dom (Zxyz);

		calib_convert_cpu_time (T1 - T0, tbuf);
		printf ("\nCPU seconds:\t%s\n", tbuf);
	}
	exit (0);
}
