/**********************************************************************

	$Id: test_zxyz_gcd_n.c,v 1.1 2024/10/20 01:34:43 warme Exp $

	File:	test_zxyz_gcd_n.c
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2012, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	Test Z[x,y,z] gcd_n().

***********************************************************************

	Modification Log:

	a-1:	09/03/2015	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#include "calib/cputime.h"
#include "calib/genrep.h"
#include "calib/logic.h"
#include "calib/new.h"
#include "calib/prompt.h"
#include "calib/Zxyz.h"
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

	int
main ()

{
int				i, npoly;
int				status;
struct calib_genrep *		gp1;
struct calib_genrep *		gp2;
const struct calib_Zxyz_dom *	ring;
struct calib_genrep **		garray;
struct calib_Zxyz_obj *		parray;
struct calib_Zxyz_obj *		cfarray;
const struct calib_Zxyz_obj **	pptrs;
struct calib_Zxyz_obj **	cfptrs;
calib_cpu_time_t		T0, T1;
char				tbuf [32];
struct calib_genrep_varlist	varlist;
struct calib_Zxyz_obj		POLY;
struct calib_Zxyz_obj		GCD;
char				buf [64];

	scanf (" %d", &npoly);

	if (npoly < 2) {
		printf ("Need at least 2 polynomials!\n");
		exit (1);
	}

	garray = CALIB_NEWA (npoly, struct calib_genrep *);
	for (i = 0; i < npoly; i++) {
		sprintf (buf, "Polynomial %d: ", i);
		calib_prompt (buf);
		gp1 = calib_genrep_read (&status);
		if (status NE 0) {
			fprintf (stderr, "Syntax error\n");
			exit (1);
		}
		if (gp1 EQ NULL) {
			exit (1);
		}
		garray [i] = gp1;
	}

	/* Crude but simple way to get combined varlist... */
	gp1 = calib_genrep_add2 (garray [0], garray [1]);
	for (i = 2; i < npoly; i++) {
		gp2 = calib_genrep_add2 (gp1, garray [i]);
		calib_genrep_free (gp1);
		gp1 = gp2;
	}
	calib_genrep_get_varlist (&varlist, gp1);
	calib_genrep_free (gp1);

	/* Make the multi-variate domain. */
	ring = calib_make_Zxyz_dom (varlist.nvars, varlist.vlist);

	calib_genrep_clear_varlist (&varlist);

	/* Init buffers for polynomials and their cofactors. */
	parray = CALIB_NEWA (npoly, struct calib_Zxyz_obj);
	cfarray = CALIB_NEWA (npoly, struct calib_Zxyz_obj);
	for (i = 0; i < npoly; i++) {
		ring -> init (ring, &parray [i]);
		ring -> init (ring, &cfarray [i]);
	}

	ring -> init (ring, &POLY);
	ring -> init (ring, &GCD);

	/* Convert all polynomials into Zxyz form. */
	for (i = 0; i < npoly; i++) {
		gp1 = garray [i];
		garray [i] = NULL;
		ring -> set_genrep (&POLY, gp1);
		calib_genrep_free (gp1);
		ring -> set (&parray [i], &POLY);
	}
	free (garray);

#if 0
	for (i = 0; i < npoly; i++) {
		printf ("\nOriginal poly %d =\n", i);
		gp1 = ring -> togenrep (&parray [i]);
		calib_genrep_print (gp1);
		calib_genrep_free (gp1);
	}
#endif

	pptrs = CALIB_NEWA (npoly, const struct calib_Zxyz_obj *);
	cfptrs = CALIB_NEWA (npoly, struct calib_Zxyz_obj *);

	for (i = 0; i < npoly; i++) {
		pptrs [i]  = &parray [i];
		cfptrs [i] = &cfarray [i];
	}

	T0 = calib_get_cpu_time ();
	ring -> gcd_n (&GCD, cfptrs, npoly, pptrs);
	T1 = calib_get_cpu_time ();

	gp1 = ring -> to_genrep (&GCD);

	ring -> clear (&GCD);

	printf ("\nGCD =\n");
	calib_genrep_print (gp1);
	printf ("\n");

	calib_genrep_free (gp1);

	for (i = 0; i < npoly; i++) {
		printf ("\nReduced poly %d =\n", i);
		gp1 = ring -> to_genrep (&cfarray [i]);
		calib_genrep_print (gp1);
		calib_genrep_free (gp1);
	}

	free (cfptrs);
	free (pptrs);

	ring -> clear (&GCD);
	ring -> clear (&POLY);

	for (i = 0; i < npoly; i++) {
		ring -> clear (&cfarray [i]);
		ring -> clear (&parray [i]);
	}
	free (cfarray);
	free (parray);

	calib_free_Zxyz_dom (ring);

	calib_convert_cpu_time (T1 - T0, tbuf);
	printf ("\nCPU seconds:\t%s\n", tbuf);

	exit (0);
}
