/**********************************************************************

	$Id: test_func_multiarg.c,v 1.1 2024/10/20 00:43:01 warme Exp $

	File:	test_func_multiarg.c
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2012, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	Test genrep printing of long, multi-argument functions.

***********************************************************************

	Modification Log:

	a-1:	08/06/2012	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#include "calib/fatal.h"
#include "calib/genrep.h"
#include "calib/logic.h"
#include "calib/prompt.h"
#include "calib/Zx.h"
#include "calib/Zxyz.h"
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

	int
main (

int		argc,
char **		argv
)
{
int				i, n;
calib_bool			old_printer;
char *				me;
char *				ap;
struct calib_genrep_list *	root;
struct calib_genrep_list **	hookp;
struct calib_genrep_list *	lp;
struct calib_genrep *		gp1;
const struct calib_Zx_dom *	Zx;
struct calib_Zx_obj		PROD, POLY;

	me = argv [0];

	old_printer = FALSE;

	ap = NULL;
	if (argc >= 2) {
		ap = argv [1];
		if ((ap [0] EQ '-') AND (ap [1] EQ 'o')) {
			old_printer = TRUE;
			ap = NULL;
			if (argc >= 3) {
				ap = argv [2];
			}
		}
	}
	n = 12;
	if (ap NE NULL) {
		n = atoi (ap);
		if (n < 0) {
			fprintf (stderr,
				 "%s: Negative power not permitted.\n",
				 me);
			exit (1);
		}
	}

	/* Set Poly = x + 1. */
	Zx = calib_get_Zx_dom ();
	Zx -> init (&PROD);
	Zx -> init (&POLY);
	Zx -> set_var_power (&POLY, 1);
	mpz_set_ui (&POLY.coeff [0], 1);

	Zx -> set_si (&PROD, 1);
	
	/* Generate long list of (increasingly long) arguments. */
	root	= NULL;
	hookp	= &root;
	for (i = 0; i < n; i++) {
		Zx -> mul (&PROD, &PROD, &POLY);
		gp1 = Zx -> to_genrep (&PROD, "x");
		lp = calib_genrep_new_list (gp1, NULL);
		*hookp	= lp;
		hookp	= &(lp -> next);
	}

	gp1 = calib_genrep_var ("foobar");
	gp1 = calib_genrep_func (gp1, root);

	printf ("\n");
	if (old_printer) {
		calib_genrep_print (gp1);
	}
	else {
		calib_genrep_prettyprint (gp1);
	}
	printf ("\n");

	calib_genrep_free (gp1);

	Zx -> clear (&POLY);
	Zx -> clear (&PROD);

	exit (0);
}
