/**********************************************************************

	$Id: test_pp.c,v 1.1 2024/10/20 00:43:02 warme Exp $

	File:	test_pp.c
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2012, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	Test genrep prettyprint().

***********************************************************************

	Modification Log:

	a-1:	08/06/2012	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#include "calib/genrep.h"
#include "calib/logic.h"
#include "calib/prompt.h"
#include "calib/Zxyz.h"
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

	int
main (

int		argc,
char **		argv
)
{
int				status, expand;
const char *			me;
const char *			ap;
struct calib_genrep *		gp1;
const struct calib_Zxyz_dom *	Zxyz;
struct calib_genrep_varlist	varlist;
struct calib_Zxyz_obj		POLY;

	me = argv [0];

	expand = FALSE;
	if (argc > 1) {
		ap = argv [1];
		if ((ap [0] EQ '-') AND (ap [1] EQ 'x') AND (ap [2] EQ '\0')) {
			expand = TRUE;
		}
		else {
			fprintf (stderr,
				 "Usage: %s [-x]\n"
				 "Pretty-print expressions read from stdin.\n"
				 "The -x flag expands before printing.\n",
				 me);
			exit (1);
		}
	}

	while (1) {
		calib_prompt ("Polynomial: ");
		gp1 = calib_genrep_read (&status);
		if (status NE 0) {
			fprintf (stderr, "Syntax error\n");
			continue;
		}
		if (gp1 EQ NULL) break;

		calib_genrep_get_varlist (&varlist, gp1);

		/* Make the multi-variate domain. */
		Zxyz = calib_make_Zxyz_dom (varlist.nvars, varlist.vlist);

		calib_genrep_clear_varlist (&varlist);

		if (expand) {
			Zxyz -> init (Zxyz, &POLY);

			Zxyz -> set_genrep (&POLY, gp1);
			calib_genrep_free (gp1);

			gp1 = Zxyz -> to_genrep (&POLY);
			Zxyz -> clear (&POLY);
		}

		printf ("\n");
		calib_genrep_prettyprint (gp1);
		printf ("\n");

		calib_genrep_free (gp1);

		calib_free_Zxyz_dom (Zxyz);
	}
	exit (0);
}
