/**********************************************************************

	$Id: test_rat1.c,v 1.1 2024/10/20 01:02:51 warme Exp $

	File:	test_rat1.c
	Rev:	a-2
	Date:	10/19/2024

	Copyright (c) 2020, 2024 by David M. Warme.  All rights
	reserved.  This work is licensed under a Creative Commons
	Attribution-NonCommercial-ShareAlike 4.0 International License.

***********************************************************************

	Test driver to debug a specific problem instance.

***********************************************************************

	Modification Log:

	a-1:	07/11/2020	warme
		: Created.
	a-2:	10/19/2024	warme
		: Cleanup for initial release.

***********************************************************************/

#include "calib/calib.h"
#include "calib/cputime.h"
#include "calib/logic.h"
#include <stddef.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

	int
main ()

{
const struct calib_Zxyz_dom *	dom;
const struct calib_rat_dom *	rat;
const char *			vars [2];
struct calib_genrep *		gp1;
struct calib_genrep *		gp2;
struct calib_genrep *		gp3;
struct calib_genrep *		gp4;
struct calib_genrep *		gp5;
struct calib_Zxyz_obj		TEMP1, TEMP2;
struct calib_rat_obj		COEFF;

	vars [0] = "x";
	vars [1] = "t";

	dom = calib_make_Zxyz_dom (2, vars);
	rat = calib_make_rat_dom (dom);

	dom -> init (dom, &TEMP1);
	dom -> init (dom, &TEMP2);

	rat -> init (rat, &COEFF);

	/* Generate TEMP1 = x^38 in Z[x,t]. */
	dom -> alloc (&TEMP1, 1);
	memset (TEMP1.pow, 0, 2*sizeof (int));
	TEMP1.pow [0] = 38;
	mpz_set_ui (&TEMP1.coeff [0], 1);
	TEMP1.nterms = 1;

	/* Generate TEMP2 = x^16 - x^8 + x^4 - 1 in Z[x,t]. */
	dom -> alloc (&TEMP2, 4);
	memset (TEMP2.pow, 0, 8 * sizeof (int));
	TEMP2.pow [0] = 16;
	TEMP2.pow [2] = 8;
	TEMP2.pow [4] = 4;
	TEMP2.pow [6] = 0;
	mpz_set_si (&TEMP2.coeff [0], 1);
	mpz_set_si (&TEMP2.coeff [1], -1);
	mpz_set_si (&TEMP2.coeff [2], 1);
	mpz_set_si (&TEMP2.coeff [3], -1);
	TEMP2.nterms = 4;

	gp1 = dom -> to_genrep (&TEMP1);
	gp2 = dom -> to_genrep (&TEMP2);

	/* Numerator = - TEMP2**2. */
	gp3 = calib_genrep_ipow (gp2, 2);
	gp3 = calib_genrep_neg (gp3);

	/* Denominator = x**38 * TEMP2**10. */

	gp4 = calib_genrep_ipow (gp2, 10);
	gp4 = calib_genrep_mul2 (gp1, gp4);

	gp5 = calib_genrep_div2 (gp3, gp4);
	calib_genrep_prettyprint (gp5);

	dom -> set_genrep (&COEFF.numer, gp3);
	dom -> set_genrep (&COEFF.denom, gp4);

	calib_genrep_free (gp5);
	calib_genrep_free (gp4);
	calib_genrep_free (gp3);
	calib_genrep_free (gp2);
	calib_genrep_free (gp1);

	printf ("\n");
	rat -> print (&COEFF);

	/* Demonstrate the problem. */
	rat -> canonicalize (&COEFF);

	printf ("\n");
	rat -> print (&COEFF);

	rat -> clear (&COEFF);

	dom -> clear (&TEMP2);
	dom -> clear (&TEMP1);

	calib_free_rat_dom (rat);
	calib_free_Zxyz_dom (dom);

	calib_shutdown ();

	exit (0);
}
